/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * AlarmYearWithSiecle
 *     The year for the date is now with 4 digits.
 *     You MUST specify this option to use the reminders.
 *
 * AlarmReminderBefore n
 *     To activate the alarm n days before the time on LiteStep start.
 *     You must specify the 'r' flag to enable this feature on the alarm.
 *     
 * AlarmReminderAfter n
 *     To activate the alarm n days after the time on LiteStep start.
 *     You must specify the 'r' flag to enable this feature on the alarm.
 *     
 * AlarmReminderEveryMinute n
 *    To activate the alarm every n minutes between AlarmReminderBefore and AlarmReminderAfter.
 * 
 * AlarmNoUSTime
 *     Range for hour is 0 to 23 instead of 0 to 12 (with AM/PM).
 *
 * *Alarm - execute the command at the time/date
 * *Alarm time [date] [mode] [flags] command [args]
 * - time:     time in format hh:mm with hh from 0 to 12 and AM or PM after.
 *             if AlarmNoUSTime is define, hh if from 0 to 23, without AM/PM.
 * - date:     (optional) set a date for the alarm.
 * - mode:     (optional) can be everyday, weekly, monthly, yearly or once.
 *             By default, mode is everyday.
 *             If a date is specify, the default mode is once.
 *            This option is useful if you specify a date, or else...  ^_^;
 * - flags:    #n : specify a group n
 *             d  : disable this alarm (enable it with !EnableAlarm)
 *             r  : enable AlarmReminderBefore and AlarmReminderAfter on this alarm
 *             i  : enable AlarmReminderEveryMinute on this alarm
 *                 (create an AlarmInt for you  =)  )
 * example:
 *     *Alarm 1:00AM "C:\program files\winamp\winamp.exe" c:\mp3s\
 *     this will execute winamp at 1am
 *     *Alarm 20:40 16/02/01 !BirthdayLabelShow
 *    this will show my label named BirthdayLabel at 8:40pm the 16/02/01
 *     *Alarm 13:00 1/02/01 monthly #2 !msgbox "Scan your hard drives !!!"
 *     every month, starting the 1/02/01 at 13:00, this will display a message box.
 * 		 
 * *AlarmInt - will execute a command every however minutes
 * *AlarmInt minutes [flags] command
 * - flags:    #n : specify a group n
 *             d  : disable this alarm (enable it with !EnableAlarm)
 * example:
 * 	*AlarmInt 5 "C:\4dos\4dos.com"
 *	this will execute 4dos every 5 minutes (don't know why you'd want to =)
 * 	*AlarmInt 15 !msgbox "do your homeworks instead of playing Quake 3 !!!"
 * 	maybe more useful  ^_^;
 * 
 * bang commands:
 * !ToggleAlarm [n]- toggles alarms on/off, if n is specify, toggle the group of alarms
 * !DisableAlarm [n] - disables alarms, if n is specify, disable the group of alarms
 * !EnableAlarm [n] - enables the alarms, if n is specify, enable the group of alarms
 * !AlarmStat - shows some status information
 *
 * please email me with comments/suggestions/bugs
 * - pod (danielb@vnet.net)
 */

#include <windows.h>
#include "../lsapi/safestr.h"
#include <string.h>
#include <time.h>
#include "alarm.h"

using namespace std;

const char RCS_REVISION[] = "$Revision: 1.6 (Syl) $"; // Our Version 
const char RCS_ID[] = "$Id: alarm.c,v 1.6 2002/01/11 pod modified by Syl Exp $"; // The Full RCS ID.
const char *date_mode[5] = {"everyday", "weekly", "monthly", "yearly", "once"};
const int	day_in_month[13] = {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

char szAppName[] = "AlarmModule";
CAlarmStorage* alarmStorage;

/************************************************************************************/

CAlarmStorage::CAlarmStorage(HWND parentWnd, int& code):
Window(szAppName),
alarms(NULL), curAlarm(NULL),
alarmsInt(NULL), curAlarmInt(NULL),
is_enabled(true),
USTime(true),
YearWithSiecle(false),
reminderBefore(0),
reminderAfter(0),
reminderEveryMinute(0)
{
  if (!createWindow(WS_EX_TOOLWINDOW, szAppName, WS_POPUP, 0, 0, 0, 0, parentWnd))
  {
    code = 1;
    return;
  }
	createAlarms();
	code = 0;
}

/************************************************************************************/

CAlarmStorage::~CAlarmStorage()
{
	destroyWindow();
}

/************************************************************************************/

void CAlarmStorage::createAlarms()
{
  char configLine[MAX_LINE_LENGTH];
	FILE *f;

  GetRCString("AlarmReminderBefore", configLine, "0", MAX_LINE_LENGTH);
	reminderBefore = atoi(configLine);
  GetRCString("AlarmReminderAfter", configLine, "0", MAX_LINE_LENGTH);
	reminderAfter = atoi(configLine);
  GetRCString("AlarmReminderEveryMinute", configLine, "0", MAX_LINE_LENGTH);
	reminderEveryMinute = atoi(configLine);
	USTime = GetRCBool("AlarmNoUSTime", false);
	YearWithSiecle = GetRCBool("AlarmYearWithSiecle", true);
	if ((reminderBefore || reminderAfter) && !YearWithSiecle)
	{
		reminderBefore = 0;
		reminderAfter = 0;
		reminderEveryMinute = 0;
	}

	f = LCOpen(NULL);
	while(LCReadNextConfig(f, "*Alarm", configLine, MAX_LINE_LENGTH))
	{
		if (curAlarm)
		{
			curAlarm->next = new CAlarm(this, configLine);
			curAlarm = curAlarm->next;
		}
		else
		{
			alarms = new CAlarm(this, configLine);
			curAlarm = alarms;
		}
	}
	LCClose(f);

	f = LCOpen(NULL);
	while(LCReadNextConfig(f, "*AlarmInt", configLine, MAX_LINE_LENGTH))
	{
		if (curAlarmInt)
		{
			curAlarmInt->next = new CAlarmInt(configLine);
			curAlarmInt = curAlarmInt->next;
		}
		else
		{
			alarmsInt = new CAlarmInt(configLine);
			curAlarmInt = alarmsInt;
		}
		SetTimer(hWnd, curAlarmInt->timerid, curAlarmInt->intr, NULL);
	}
	LCClose(f);
}

/************************************************************************************/

void CAlarmStorage::destroyAlarms()
{
	while (alarms)
	{
		curAlarm = alarms->next;
		delete alarms;
		alarms = curAlarm;
	}

	while (alarmsInt)
	{
		curAlarmInt = alarmsInt->next;
		delete alarmsInt;
		alarmsInt = curAlarmInt;
	}
}

/************************************************************************************/

void CAlarmStorage::windowProc(Message& message)
{
  BEGIN_MESSAGEPROC
    MESSAGE(onCreate,   WM_CREATE)
    MESSAGE(onDestroy,  WM_DESTROY)
    MESSAGE(onTimer,	  WM_TIMER)
    MESSAGE(onGetRevId, LM_GETREVID)
    MESSAGE(onRefresh,	LM_REFRESH)
  END_MESSAGEPROC
}

/************************************************************************************/

void CAlarmStorage::onCreate(Message& message)
{
  int lsMessages[] = {LM_GETREVID, LM_REFRESH, 0};

  SendMessage(hParent, LM_REGISTERMESSAGE, WPARAM(hWnd), LPARAM(lsMessages));

	SetWindowLong(hWnd, GWL_USERDATA, magicDWord);

	AddBangCommand("!disablealarm", BangDisableAlarm);
	AddBangCommand("!enablealarm", BangEnableAlarm);
	AddBangCommand("!togglealarm", BangToggleAlarm);
	AddBangCommand("!alarmstat", BangAlarmStat);
	SetTimer(hWnd, 1, 1000, NULL);
}

/************************************************************************************/

void CAlarmStorage::onDestroy(Message& message)
{
  int lsMessages[] = {LM_GETREVID, LM_REFRESH, 0};

	destroyAlarms();
	RemoveBangCommand("!disablealarm");
	RemoveBangCommand("!enablealarm");
	RemoveBangCommand("!togglealarm");
	RemoveBangCommand("!alarmstat");

  SendMessage(hParent, LM_UNREGISTERMESSAGE, WPARAM(hWnd), LPARAM(lsMessages));
}

/************************************************************************************/

void CAlarmStorage::onGetRevId(Message& message)
{
  char* buf = (char*)(message.lParam);

  switch (message.wParam)
  {
    case 0:
      sprintf(buf, "alarm.dll: %s", &RCS_REVISION[11]);
      buf[StrLen(buf) - 1] = '\0';
      break;
    case 1:
      StrCopy(buf, &RCS_ID[1]);
      buf[StrLen(buf) - 1] = '\0';
      break;
    default:
      buf[0] = 0;
  }
  message.lResult = StrLen(buf);
}

/************************************************************************************/

void CAlarmStorage::onRefresh(Message& message)
{
	destroyAlarms();
	createAlarms();
}

/************************************************************************************/

void CAlarmStorage::BangEnable(LPCTSTR args)
{
	if (!StrLen(args))
		is_enabled = true;
	else
	{
		curAlarm = alarms;
		while (curAlarm)
		{
			if (curAlarm->group == atoi(args))
				curAlarm->is_enabled = true;
			curAlarm = curAlarm->next;
		}
		curAlarmInt = alarmsInt;
		while (curAlarmInt)
		{
			if (curAlarmInt->group == atoi(args))
				curAlarmInt->is_enabled = true;
			curAlarmInt = curAlarmInt->next;
		}
	}
}

void CAlarmStorage::BangDisable(LPCTSTR args)
{
	if (!StrLen(args))
		is_enabled = false;
	else
	{
		curAlarm = alarms;
		while (curAlarm)
		{
			if (curAlarm->group == atoi(args))
				curAlarm->is_enabled = false;
			curAlarm = curAlarm->next;
		}
		curAlarmInt = alarmsInt;
		while (curAlarmInt)
		{
			if (curAlarmInt->group == atoi(args))
				curAlarmInt->is_enabled = false;
			curAlarmInt = curAlarmInt->next;
		}
	}
}

void CAlarmStorage::BangToggle(LPCTSTR args)
{
	if (!StrLen(args))
		is_enabled = !is_enabled;
	else
	{
		curAlarm = alarms;
		while (curAlarm)
		{
			if (curAlarm->group == atoi(args))
				curAlarm->is_enabled = !curAlarm->is_enabled;
			curAlarm = curAlarm->next;
		}
		curAlarmInt = alarmsInt;
		while (curAlarmInt)
		{
			if (curAlarmInt->group == atoi(args))
				curAlarmInt->is_enabled = !curAlarmInt->is_enabled;
			curAlarmInt = curAlarmInt->next;
		}
	}
}

void CAlarmStorage::BangStat(LPCTSTR args)
{
	char buffer[MAX_LINE_LENGTH];
	char buf[1024];
	time_t tVal;
	struct tm *stVal;
	int i;

	updateTime(true);
	memset(buffer, 0, MAX_LINE_LENGTH);

	sprintf(buffer, "current time: %s, current date: %s\n", m_time, m_date);

	if (is_enabled)
		strcat(buffer, "All alarms are enabled\n");
	else
		strcat(buffer, "All alarms are disabled !!!\n");

	if (!(reminderBefore || reminderAfter))
		strcat(buffer, "Reminders are disabled, reminders flags on alarms are disabled (did you forget the AlarmYearWithSiecle ???)\n");
	else
		strcat(buffer, "Reminders are enabled\n");

	curAlarm = alarms;
	while (curAlarm)
	{
		sprintf(buf, "<Alarm> time: %s", curAlarm->m_time);
		if (curAlarm->mode)
		{
			strcat(buffer, buf);
			if (curAlarm->mode != 4)
				sprintf(buf, ", date: %s, mode: %s", curAlarm->m_date, date_mode[curAlarm->mode]);
			else
				sprintf(buf, ", date: %s", curAlarm->m_date);
		}
		if (curAlarm->group)
		{
			strcat(buffer, buf);
			sprintf(buf, ", group: %d", curAlarm->group);
			if (curAlarm->reminder)
				strcat(buf, "  [Reminder enable]");
		}
		if (curAlarm->is_enabled)
				strcat(buf, "  [enable]\n");
		else
				strcat(buf, "  [disable]\n");
		strcat(buffer, buf);
		sprintf(buf, "--> Command: \"%s\"", curAlarm->command);
		strcat(buffer, buf);
		if (StrLen(curAlarm->args))
		{
			sprintf(buf, " \"%s\"\n", curAlarm->args);
			strcat(buffer, buf);
		}
		else
			strcat(buffer, "\n");
		curAlarm = curAlarm->next;
	}
	curAlarmInt = alarmsInt;
	while (curAlarmInt)
	{
		sprintf(buf, "<AlarmInt> timer: %d minutes", curAlarmInt->intr / 60000, curAlarmInt->command);
		if (curAlarmInt->group)
		{
			strcat(buffer, buf);
			sprintf(buf, ", group: %d", curAlarmInt->group);
		}
		if (curAlarmInt->is_enabled)
				strcat(buf, "  [enable]\n");
		else
				strcat(buf, "  [disable]\n");
		strcat(buffer, buf);
		sprintf(buf, "--> Command: \"%s\"", curAlarmInt->command);
		strcat(buffer, buf);
		if (StrLen(curAlarmInt->args))
		{
			sprintf(buf, " \"%s\"\n", curAlarmInt->args);
			strcat(buffer, buf);
		}
		else
			strcat(buffer, "\n");
		curAlarmInt = curAlarmInt->next;
	}
	MessageBox(hWnd, buffer, szAppName, MB_OK|MB_TOPMOST);
}

/************************************************************************************/

void BangEnableAlarm(HWND sender, LPCTSTR args)
{
	alarmStorage->BangEnable(args);
}

void BangDisableAlarm(HWND sender, LPCTSTR args)
{
	alarmStorage->BangDisable(args);
}

void BangToggleAlarm(HWND sender, LPCTSTR args)
{
	alarmStorage->BangToggle(args);
}

void BangAlarmStat(HWND sender, LPCTSTR args)
{
	alarmStorage->BangStat(args);
}

/************************************************************************************/

void CAlarmStorage::onTimer(Message& message)
{
	if (!is_enabled) return;

	if(message.wParam == 1)
		updateAlarms();
	else
	{
		curAlarmInt = alarmsInt;
		while (curAlarmInt)
		{
			if((int)message.wParam == curAlarmInt->timerid)
			{
				curAlarmInt->exec(hWnd);
				return;
			}
			curAlarmInt = curAlarmInt->next;
		}
		curAlarm = alarms;
		while (curAlarm)
		{
			if((int)message.wParam == curAlarm->ktimer)
			{
				KillTimer(hWnd, curAlarm->ktimer);
				curAlarm->is_enabled = true;
			}
			curAlarm = curAlarm->next;
		}
	}

}

/************************************************************************************/

bool CAlarmStorage::updateTime(bool force)
{
	time_t tVal;
	struct tm *stVal;

	time(&tVal);

	stVal = localtime(&tVal);
	/*if (!force)					// it was to optimize a little...
	{
		strftime(m_time, 2, "%S", stVal);
		if (!StrIPos(m_time, "00"))
			return false;
	}*/
	if (USTime)
		strftime(m_time, 10, "%I:%M%p", stVal);
	else
		strftime(m_time, 10, "%H:%M", stVal);
	if(m_time[0] == '0')
		StrCopy(m_time, &m_time[1]); // get rid of leading 0
	sscanf(m_time, "%d%*c%d", &hour, &min);

	if (YearWithSiecle)
		strftime(m_date, 15, "%d/%m/%Y", stVal);
	else
		strftime(m_date, 10, "%d/%m/%y", stVal);
	if(m_date[0] == '0')
		StrCopy(m_date, &m_date[1]); // get rid of leading 0
	sscanf(m_date, "%d%*c%d%*c%d", &day, &month, &year);
	return true;
}

/************************************************************************************/

bool CAlarmStorage::testDate(CAlarm* alarm)
{
	int d, m, y, sum = 0;
	int d2, m2, y2;
	time_t tVal;
	struct tm *stVal;
	char str[4];
	char buffer[1024];
	bool time_ok = stricmp(m_time, alarm->m_time) == 0;

	if (alarm->reminder)
	{
		switch (alarm->mode) {
			case 0:
				return (alarm->evalDate(m_date));
				break;
			case 1:
				return (alarm->evalDate(m_date));
				break;
			case 2:
				sscanf(m_date, "%d%*c%d%*c%d", &d, &m, &y);
				sscanf(alarm->m_date, "%d%*c%d%*c%d", &d2, &m2, &y2);
				sprintf(buffer, "%d/%02d/%04d", d, m2, y2);
				return (alarm->evalDate(buffer));
				break;
			case 3:
				sscanf(m_date, "%d%*c%d%*c%d", &d, &m, &y);
				sscanf(alarm->m_date, "%d%*c%d%*c%d", &d2, &m2, &y2);
				sprintf(buffer, "%d/%02d/%04d", d, m, y2);
				return (alarm->evalDate(buffer));
				break;
			case 4:
				return (alarm->evalDate(m_date));
				break;
			}
	}
	else
	{
		switch (alarm->mode) {
			case 0:
				return time_ok;
			case 1:
				sscanf(alarm->m_date, "%d%*c%d%*c%d", &d, &m, &y);
				if ((y & 3 == 0) && (m & 0xC))		// or (y % 4 == 0) && (m >= 2)
					sum++;
				sum += d;
				for (d = 1; d < m; d++)
					sum += day_in_month[d];

				time(&tVal);

				stVal = localtime(&tVal);
				strftime(str, 4, "%j", stVal);

				if (sum % 7 == atoi(str) % 7)
					return time_ok;
				break;
			case 2:
				sscanf(alarm->m_date, "%d%*c", &d);
				if (d == day) return time_ok;
				break;
			case 3:
				sscanf(alarm->m_date, "%d%*c%d%*c", &d, &m);
				if ((d == day) && (m == month)) return time_ok;
				break;
			case 4:
				return((stricmp(m_date, alarm->m_date) == 0) && time_ok);
		}
	}
	return false;
}

/************************************************************************************/

void CAlarmStorage::updateAlarms()
{

	if(!(is_enabled && updateTime(false))) return;

	curAlarm = alarms;
	while (curAlarm)
	{
		if (testDate(curAlarm))
			curAlarm->exec(hWnd);

		curAlarm = curAlarm->next;
	}
}

/************************************************************************************/
/************************************************************************************/
/************************************************************************************/

int CAlarm::id = 0;

/************************************************************************************/

CAlarm::CAlarm(CAlarmStorage* o, char* line):
is_enabled(true),
reminder(false),
mode(0),
group(0),
next(NULL)
{
	my_id = id++;
	owner = o;
	init(line);
}

/************************************************************************************/

void CAlarm::init(char* line)
{
	char token[1024];
	LPCSTR nextToken = line;
	bool hasCommand = true;
	int i;
	time_t tVal;
	struct tm *stVal;

	is_enabled = true;

	GetToken(nextToken, NULL, &nextToken, false);
	if (GetToken(nextToken, token, &nextToken, false))
	{
		StrLCopy(m_time, token, 10);
		if(m_time[0] == '0')
			StrCopy(m_time, &m_time[1]);
	}

	if (GetToken(nextToken, token, &nextToken, false))
	{
		if (CharInStr('/', token))
		{
			StrLCopy(m_date, token, 15);
			if(m_date[0] == '0')
				StrCopy(m_date, &m_date[1]);
			mode = 4;
			hasCommand = GetToken(nextToken, token, &nextToken, false);
		}
		else
		{
			mode = 0;

			time(&tVal);

			stVal = localtime(&tVal);
			
			if (owner->YearWithSiecle)
				strftime(m_date, 15, "%d/%m/%Y", stVal);
			else
				strftime(m_date, 15, "%d/%m/%y", stVal);
			if(m_date[0] == '0')
				StrCopy(m_date, &m_date[1]);
		}

		if (hasCommand)
		{
			mode = -mode;

			for (i = 0; i <= 4; i++)
				if (StrIPos(token, date_mode[i]))
					mode = i;

			if (mode >= 0)
				hasCommand = GetToken(nextToken, token, &nextToken, false);
			else
				mode = -mode;
		}

		i = 0;
		if (token[0] == '#')
		{
			strlwr(token);
			group						= atoi(&token[1]);
			is_enabled			=!CharInStr('d', token);
			if (owner && (owner->reminderBefore != 0) || (owner->reminderAfter != 0))
			{
				reminder			= CharInStr('r', token);
				if (reminder)
				{
					initDate(date_before, owner->reminderBefore, -1);
					initDate(date_after, owner->reminderAfter, 1);
				}
				else
				{
					initDate(date_before, 0, 0);
					initDate(date_after, 0, 0);
				}
				reminder_time				=	CharInStr('i', token);
			}
			hasCommand =		GetToken(nextToken, token, &nextToken, false);
		}

		if (hasCommand)
			StrLCopy(command, token, 1024);
	}

	StrLCopy(args, nextToken, 1024);
}

/************************************************************************************/

void CAlarm::initDate(char* tab, int r, int s)
{
	int d, m, y;
	int a, b, c;

	m = y = 0;

	if (r == 0)
		StrCopy(tab, m_date);
	else
	{
		sscanf(m_date, "%d%*c%d%*c%d%", &d, &m, &y);
		while (r >= 365 + ((y + s) & 3 == 0))
		{
			y += s;
			r -= 365 + ((y + 1) & 3 == 0);
		}
		while (r > day_in_month[1 + (m - 1 + s) % 12])
		{
			m += s;
			if (m < 1)
			{
				m = 12;
				y--;
			}
			if (m > 12)
			{
				m = 1;
				y++;
			}
			r -= day_in_month[m];
		}
		d += s * r;
		if (d <= 0)
		{
			m += s;
			if (m < 1)
			{
				m = 12;
				y--;
			}
			d = day_in_month[m] - d;
		}
		if (d > day_in_month[m])
		{
			d -= day_in_month[m];
			m += s;
			if (m > 12)
			{
				m = 1;
				y++;
			}
		}
		sprintf(tab, "%d/%02d/%04d", d, m, y);
	}

	sscanf(date_before,	"%d%*c%d%*c%d", &a, &b, &c);
	sprintf(computed_date[0], "%04d%02d%02d", c, b, a);
	sscanf(date_after,	"%d%*c%d%*c%d", &a, &b, &c);
	sprintf(computed_date[2], "%04d%02d%02d", c, b, a);
}

/************************************************************************************/

void CAlarm::exec(HWND hWnd)
{
	if (!is_enabled) return;
	if (reminder)
		reminder = false;
	else
		is_enabled = false;
	ktimer = (my_id + 100);
	SetTimer(hWnd, ktimer, 60000, NULL);
	if(command[0] == '!')
		ParseBangCommand(hWnd, command, args);
	else
		ShellExecute(hWnd, "open", command, args, NULL, SW_SHOWDEFAULT);

	if (reminder_time && owner->reminderEveryMinute)
	{
		reminder_time = false;
		char configLine[1024];
		if (group)
			sprintf(configLine, "*AlarmInt %d #%d \"%s\" \"%s\"", owner->reminderEveryMinute, group, command, args);
		else
			sprintf(configLine, "*AlarmInt %d \"%s\" \"%s\"", owner->reminderEveryMinute, command, args);
		if (owner->alarmsInt)
		{
			owner->curAlarmInt->next = new CAlarmInt(configLine);
			owner->curAlarmInt = owner->curAlarmInt->next;
		}
		else
		{
			owner->alarmsInt = new CAlarmInt(configLine);
			owner->curAlarmInt = owner->alarmsInt;
		}
	}
}

/************************************************************************************/

bool CAlarm::evalDate(char* g_date)
{
	int a, b, c;

	if (!reminder)
		return false;

	sscanf(g_date,	"%d%*c%d%*c%d", &a, &b, &c);
	sprintf(computed_date[1], "%04d%02d%02d", c, b, a);
	return ((stricmp(computed_date[0], computed_date[1]) <= 0) && (stricmp(computed_date[1], computed_date[2]) <= 0));
};

/************************************************************************************/

CAlarm::~CAlarm()
{
}

/************************************************************************************/

int CAlarmInt::id = 0;

/************************************************************************************/

CAlarmInt::CAlarmInt(char* line):
is_enabled(true),
group(0),
next(NULL)
{
	my_id = id++;
	init(line);
}

/************************************************************************************/

CAlarmInt::~CAlarmInt()
{
}

/************************************************************************************/

void CAlarmInt::init(char* line)
{
	char token[1024];
	LPCSTR nextToken = line;
	bool hasCommand = true;

	is_enabled = true;

	GetToken(nextToken, NULL, &nextToken, false);
	if (GetToken(nextToken, token, &nextToken, false))
	{
		intr = (atoi(token) * 60000);
		timerid = (my_id + 1000);
	}

	if (GetToken(nextToken, token, &nextToken, false))
	{
		if (token[0] == '#')
		{
			strlwr(token);
			group						= atoi(&token[1]);
			is_enabled			= !CharInStr('d', token);
			hasCommand			= GetToken(nextToken, token, &nextToken, false);
		}

		if (hasCommand)
			StrLCopy(command, token, 1024);
	}
	StrLCopy(args, nextToken, 1024);
}

/************************************************************************************/

void CAlarmInt::exec(HWND hWnd)
{
	if (!is_enabled) return;
	if(command[0] == '!')
		ParseBangCommand(hWnd, command, args);
	else
		ShellExecute(hWnd, "open", command, args, NULL, SW_SHOWDEFAULT);
}

/************************************************************************************/

int initModule(HWND ParentWnd, HINSTANCE dllInst, wharfDataType* wd) {
	return initModuleEx (ParentWnd, dllInst, wd->lsPath);
}

int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath) {
  int code;

  Window::init(dllInst);

  alarmStorage = new CAlarmStorage(ParentWnd, code);

  return code;
}

void quitModule(HINSTANCE dllInst) {
	delete alarmStorage;
}
